// CSC 2012
// Virtualization excercises

#include <cstdio>
#include <string>
#include "Snap.h"  // Standford Network Analysis Project (Graph library)

static void Usage(const std::string &bin_image) {
  printf("Computes the betweeness of all the vertices in a graph.\n"
         "Usage:\n"
         "  %s -g <graph file>\n\n"
         "Example:\n"
         "  %s -g power-grid.graph\n"
         "The output will be lines in the following format\n"
         "  <node id>  <betweeness>\n",
         bin_image.c_str(), bin_image.c_str());
}

int main(int argc, char **argv) {
  std::string graph_path = "";

  // Option parsing
  if (argc < 2) {
    Usage(argv[0]);
    return 1;
  }

  char c;
	while ((c = getopt(argc, argv, "g:h")) != -1) {
		switch (c) {
      case 'g':
        graph_path = optarg;
        break;
      case 'h':
      case '?':
      default:
        Usage(argv[0]);
        return 0;
    }
  }

  if (graph_path == "") {
    Usage(argv[0]);
    return 1;
  }

  PUNGraph Graph = TSnap::LoadEdgeList<PUNGraph>(graph_path.c_str());
  fprintf(stderr, "Loaded graph has %d vertices and %d edges\n",
          Graph->GetNodes(), Graph->GetEdges());

  TIntV BtwNIdV;
  for (TUNGraph::TNodeI NI = Graph->BegNI(); NI < Graph->EndNI(); NI++) {
    BtwNIdV.Add(NI.GetId());
  }

  TIntFltH NodeBtwH;
  TIntPrFltH EdgeBtwH;
  TSnap::GetBetweennessCentr(Graph, BtwNIdV, NodeBtwH, true, EdgeBtwH, false);

  for (TIntFltH::TIter CI = NodeBtwH.BegI(); CI < NodeBtwH.EndI(); CI++) {
    printf("%d %f\n", CI.GetKey()(), CI.GetDat()());
  }

  return 0;
}
